@@ -1,8 +1,8 @@
-
 expandAsMatrix <- function(x,dim=NULL,byrow=TRUE)
 #	Convert scalar, row or column vector, or matrix, to be a matrix
-#	Gordon Smyth
-#	26 Jan 2011.  Last modified 27 July 2015 (Yunshun Chen).
+#	Gordon Smyth, Yunshun Chen, Aaron Lun
+#	created 26 Jan 2011.  
+#   last modified 29 Jun 2016.
 {
 #	Check dim argument
 	if(is.null(dim)) return(as.matrix(x))
@@ -25,6 +25,11 @@ expandAsMatrix <- function(x,dim=NULL,byrow=TRUE)
 	if(length(dx)<2) stop("x has less than 2 dimensions")
 	if(length(dx)>2) stop("x has more than 2 dimensions")
 	if(all(dx==dim)) return(as.matrix(x))
+
+#	x is a CompressedMatrix
+	if(is(x, "CompressedMatrix")) {
+        return(Recall(as.matrix(x), dim=dim, byrow=byrow))
+    }
 	stop("x is matrix of wrong size")
 }
 
