@@ -1,4 +1,5 @@
 library(edgeR)
+options(warnPartialMatchArgs=TRUE,warnPartialMatchAttr=TRUE,warnPartialMatchDollar=TRUE)
 
 set.seed(0); u <- runif(100)
 
@@ -8,6 +9,8 @@ y <- rbind(0,c(0,0,2,2),y)
 rownames(y) <- paste("Tag",1:nrow(y),sep=".")
 d <- DGEList(counts=y,group=rep(1:2,each=2),lib.size=1001:1004)
 
+filterByExpr(d)
+
 # estimate common dispersion and find differences in expression
 d <- estimateCommonDisp(d)
 d$common.dispersion
@@ -28,8 +31,8 @@ summary(d2$tagwise.dispersion)
 de <- exactTest(d2)
 topTags(de)
 
-summary(exactTest(d2,rejection="smallp")$table$PValue)
-summary(exactTest(d2,rejection="deviance")$table$PValue)
+summary(exactTest(d2,rejection.region="smallp")$table$PValue)
+summary(exactTest(d2,rejection.region="deviance")$table$PValue)
 
 d2 <- estimateTagwiseDisp(d,trend="loess",span=0.8,prior.df=20)
 summary(d2$tagwise.dispersion)
@@ -51,7 +54,7 @@ lrt <- glmLRT(fit,coef=2)
 topTags(lrt)
 
 fit <- glmFit(d,design,dispersion=d$common.dispersion,prior.count=0.5)
-summary(fit$coef)
+summary(fit$coefficients)
 
 fit <- glmFit(d,design,prior.count=0.5/4)
 lrt <- glmLRT(fit,coef=2)
@@ -97,13 +100,15 @@ y <- matrix(y,ntags,nlibs)
 colnames(y) <- c("x0","x1","x2")
 rownames(y) <- paste("Gene",1:ntags,sep="")
 d <- DGEList(y)
-d <- calcNormFactors(d)
+d <- calcNormFactors(d,method="TMM")
 fit <- glmFit(d, design, dispersion=dispersion.true, prior.count=0.5/3)
 results <- glmLRT(fit, coef=2)
 topTags(results)
 d1 <- estimateGLMCommonDisp(d, design, verbose=TRUE)
 glmFit(d,design,dispersion=dispersion.true, prior.count=0.5/3)
 
+calcNormFactors(d$counts,method="TMMwsp")
+
 d2 <- estimateDisp(d, design)
 summary(d2$tagwise.dispersion)
 d2 <- estimateDisp(d, design, prior.df=20)
@@ -126,7 +131,7 @@ exactTestBetaApprox(ys$y1,ys$y2,dispersion=2/3)
 y[1,3:4] <- 0
 design <- model.matrix(~group)
 fit <- glmFit(y,design,dispersion=2/3,prior.count=0.5/7)
-summary(fit$coef)
+summary(fit$coefficients)
 
 lrt <- glmLRT(fit,contrast=cbind(c(0,1,0),c(0,0,1)))
 topTags(lrt)
